package cn.com.ty.lift.system.user.service.impl;

import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.common.utils.ProjectUtils;
import cn.com.ty.lift.system.user.dao.entity.MtCompanyUser;
import cn.com.ty.lift.system.user.dao.entity.UserApplication;
import cn.com.ty.lift.system.user.dao.entity.UserRole;
import cn.com.ty.lift.system.user.dao.entity.model.ProjectUser;
import cn.com.ty.lift.system.user.dao.entity.model.UserApplyRequest;
import cn.com.ty.lift.system.user.dao.entity.model.UserApplyResponse;
import cn.com.ty.lift.system.user.dao.entity.model.UserResponse;
import cn.com.ty.lift.system.user.dao.mapper.UserApplicationMapper;
import cn.com.ty.lift.system.user.service.*;
import cn.com.ty.lift.system.utils.CommonUtil;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 用户申请表 服务实现类
 * </p>
 *
 * @author huangyuan
 * @since 2019-12-13
 */
@Service
public class UserApplicationService extends ServiceImpl<UserApplicationMapper, UserApplication> implements IUserApplicationService {

    @Autowired
    private IMtCompanyUserService mtCompanyUserService;

    @Autowired
    private IUserRoleService userRoleService;

    @Autowired
    private IUserService userService;
    @Autowired
    private IProjectService projectService;

    @Override
    public RestResponse apply(UserApplyRequest userApplyRequest, int applyTypeTeam) {
        //获取用户原有的申请信息
        UserApplication userApplication = this.getOne(new QueryWrapper<UserApplication>()
                .eq("mt_company_id", userApplyRequest.getCompanyId())
                .eq("user_id", userApplyRequest.getUserId())
                //去掉申请未通过的判断
                .ne("status", ApiConstants.ApplicationConstants.APPLY_FAIL)
        );
        //设置前校验
        RestResponse restResponse = applyCheck(userApplyRequest, userApplication, applyTypeTeam);
        if (restResponse != null) {
            //校验不通过返回校验信息
            return restResponse;
        }
        //校验通过设置审核信息
        if (userApplication == null) {
            userApplication = new UserApplication();
            userApplication.setUserId(userApplyRequest.getUserId());
            userApplication.setMtCompanyId(userApplyRequest.getCompanyId());
            userApplication.setType(applyTypeTeam);
            //申请加入项目时需要设置的信息
            userApplication.setProjectId(userApplyRequest.getProjectId());
        }
        //审核状态变为待审核
        userApplication.setStatus(ApiConstants.ApplicationConstants.APPLY_WAIT);
        //申请加入项目时需要设置的信息
        userApplication.setPowerType(userApplyRequest.getPowerType());
        this.saveOrUpdate(userApplication);
        return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, "申请成功");
    }

    @Override
    @Transactional
    public RestResponse applyPass(UserApplyRequest userApplyRequest) {
        //加入团队审核通过
        UserApplication userApplication = this.getById(userApplyRequest.getId());
        //设置申请信息
        userApplication.setStatus(ApiConstants.ApplicationConstants.APPLY_PASS);
        userApplication.setDealDate(LocalDateTime.now());
        userApplication.setDealUserId(userApplyRequest.getUserId());
        if (ApiConstants.ApplicationConstants.APPLY_TYPE_TEAM == userApplication.getType()) {
            //团队审核通过
            teamPass(userApplyRequest, userApplication);
        } else {
            //项目审核通过
            projectPass(userApplication);
        }
        this.updateById(userApplication);
        return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, "操作成功");
    }

    @Override
    public RestResponse applyFail(UserApplyRequest userApplyRequest) {
        UserApplication userApplication = this.getById(userApplyRequest.getId());
        //设置申请状态为审核不通过
        userApplication.setStatus(ApiConstants.ApplicationConstants.APPLY_FAIL);
        this.updateById(userApplication);
        return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, "操作成功");
    }

    @Override
    public RestResponse applyList(UserApplyRequest userApplyRequest) {

        List<Long> userQueryIdList = null;
        //通过用户名或手机号查找用户id
        if (StringUtils.isNotBlank(userApplyRequest.getQueryCondition())) {
            userQueryIdList = userService.getIdListByQueryCondition(userApplyRequest.getQueryCondition());
        }

        List<UserApplyResponse> userApplyResponseList = new ArrayList<>();
        IPage userApplicationPage = new Page(userApplyRequest.getPageNum(), userApplyRequest.getPageSize());
        userApplicationPage = this.page(userApplicationPage, new QueryWrapper<UserApplication>()
                .in(userQueryIdList != null && userQueryIdList.size() > 0, "user_id", userQueryIdList)
                .eq("mt_company_id", userApplyRequest.getCompanyId())
                .eq("status", userApplyRequest.getStatus())
        );
        List<UserApplication> userApplicationList = userApplicationPage.getRecords();
        if (userApplicationList != null && userApplicationList.size() > 0) {
            //获取用id集合
            List<Long> userIdList = ProjectUtils.getAttrList(userApplicationList, "userId", null);
            //用户id对应的用户基本信息
            Map<Long, UserResponse> userIdToUserResponse = userService.getUserResponseByUserIdList(userIdList);
            for (UserApplication userApplication : userApplicationList) {
                UserApplyResponse userApplyResponse = new UserApplyResponse();
                UserResponse userResponse = userIdToUserResponse.get(userApplication.getUserId());
                //设置申请人用户信息
                if (userResponse != null) {
                    userApplyResponse.setUserAvatarUrl(userResponse.getAvatarUrl());
                    userApplyResponse.setUserName(userResponse.getName());
                    userApplyResponse.setUserMobile(userResponse.getMobile());
                    userApplyResponse.setUserType(userResponse.getType());
                }
                //设置申请基础信息
                userApplyResponse.setApplyDate(userApplication.getApplyDate());
                userApplyResponse.setApplyType(userApplication.getType());
                userApplyResponse.setPowerType(userApplication.getPowerType());
                userApplyResponse.setDealDate(userApplication.getDealDate());
                userApplyResponse.setId(userApplication.getId());
                userApplyResponse.setUserId(userApplication.getUserId());

                userApplyResponseList.add(userApplyResponse);
            }
        }
        userApplicationPage.setRecords(userApplyResponseList);
        return RestResponse.success(userApplicationPage, ApiConstants.RESULT_SUCCESS, "获取申请信息成功");
    }

    /**
     * @param userApplyRequest 申请参数
     * @param applyTypeTeam    申请类型
     * @return
     * @description 校验项目
     * @date 2020-01-05 14:14
     */
    private RestResponse applyCheck(UserApplyRequest userApplyRequest, UserApplication userApplication, int applyTypeTeam) {
        //校验用户是否已在团队或者项目中
        if (ApiConstants.ApplicationConstants.APPLY_TYPE_TEAM == applyTypeTeam) {
            //获取用户团队信息
            MtCompanyUser mtCompanyUser = mtCompanyUserService.getOne(new QueryWrapper<MtCompanyUser>()
                    .eq("mt_company_id", userApplyRequest.getCompanyId())
                    .eq("user_id", userApplyRequest.getUserId())
                    .eq("status", ApiConstants.ApplicationConstants.APPLY_FAIL)
            );
            if (mtCompanyUser != null) {
                return RestResponse.fail(ApiConstants.RESULT_ERROR, "用户已经在团队中了");
            }
        } else {
            //获取项目用户信息
            ProjectUser projectUser = projectService.getProjectUserByUserIdAndProjectId(new ProjectUser(
                    userApplyRequest.getUserId(), userApplyRequest.getCompanyId(), userApplyRequest.getProjectId()));
            if (projectUser != null) {
                return RestResponse.fail(ApiConstants.RESULT_ERROR, "用户已经在项目中了");
            }
        }
        //校验两次申请的时间
        if (userApplication != null) {
            LocalDateTime oldApplyTime = userApplication.getApplyDate();
            //当前申请时间
            LocalDateTime nowApplyTime = LocalDateTime.now();
            //差值判断
            if (CommonUtil.durationJudge(oldApplyTime, nowApplyTime, 1L)) {
                return RestResponse.success(null, ApiConstants.RESULT_ERROR, "今天已经申请过了");
            }
        }
        return null;
    }

    /**
     * @param
     * @return
     * @description 团队审核通过
     * @date 2020-01-05 15:00
     */
    private void teamPass(UserApplyRequest userApplyRequest, UserApplication userApplication) {
        //更新公司用户信息
        Long companyId = userApplication.getMtCompanyId();
        Long userId = userApplication.getUserId();
        //给用户分配角色
        UserRole userRole = new UserRole();
        userRole.setCompanyId(companyId);
        userRole.setUserId(userId);
        userRole.setRoleId(userApplyRequest.getRoleId());
        userRoleService.save(userRole);
        //获取公司用户信息
        MtCompanyUser mtCompanyUser = mtCompanyUserService.getOne(new QueryWrapper<MtCompanyUser>()
                .eq("mt_company_id", companyId)
                .eq("user_id", userId)
        );
        //如果公司用户信息不存在，新建信息
        if (mtCompanyUser == null) {
            mtCompanyUser = new MtCompanyUser();
            mtCompanyUser.setMtCompanyId(companyId);
            mtCompanyUser.setUserId(userId);
            mtCompanyUser.setCreateUserId(userId);
            mtCompanyUser.setCreateTime(new Date());
        }
        mtCompanyUser.setUpdateTime(new Date());
        mtCompanyUser.setUpdateUserId(userId);
        //设置用户状态在团队中
        mtCompanyUser.setStatus(ApiConstants.ApplicationConstants.APPLY_PASS);
        mtCompanyUserService.saveOrUpdate(mtCompanyUser);
    }

    /**
     * @param
     * @return
     * @description 团队审核通过
     * @date 2020-01-05 15:17
     */
    private void projectPass(UserApplication userApplication) {
        ProjectUser projectUser = new ProjectUser();
        projectUser.setUserId(userApplication.getUserId());
        projectUser.setCompanyId(userApplication.getMtCompanyId());
        projectUser.setProjectId(userApplication.getProjectId());
        projectUser.setUserRole(userApplication.getPowerType() + "");
        projectUser.setIsMonitor("0");
        //保存项目信息
        projectService.saveProjectUser(projectUser);
    }
}
